@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -85,6 +85,12 @@ struct sockaddr_un
 #define ESHUTDOWN ECONNABORTED
 #endif
 
+/* Windows uses non-standard names for shutdown(how). */
+#ifndef SHUT_WR
+#define SHUT_WR SD_SEND
+#define SHUT_RDWR SD_BOTH
+#endif
+
 /* This generated function translates Winsock errors into errno codes. */
 extern int translate_winsock_error (const char *fn, int err);
 
@@ -126,12 +132,13 @@ extern int win_send (int fd, const void *buf, size_t len, int flags);
  * Note: Don't use this for things which can never work on Windows
  * (eg. Unix socket support).  Those should just give regular errors.
  */
-#define NOT_IMPLEMENTED_ON_WINDOWS(feature)                             \
-  do {                                                                  \
+#define NOT_IMPLEMENTED_ON_WINDOWS(feature)                                    \
+  do {                                                                         \
     fprintf (stderr, "nbdkit: %s is not implemented for Windows.\n", feature); \
-    fprintf (stderr, "You can help by contributing to the Windows port, see\n"); \
-    fprintf (stderr, "nbdkit README in the source for how to contribute.\n"); \
-    exit (EXIT_FAILURE);                                                \
+    fprintf (stderr, "You can help by contributing to the Windows port,\n");   \
+    fprintf (stderr, "see nbdkit README in the source for how to\n");          \
+    fprintf (stderr, "contribute.\n");                                         \
+    exit (EXIT_FAILURE);                                                       \
   } while (0)
 
 #else /* !WIN32 */
