@@ -54,6 +54,10 @@
 #if CURL_AT_LEAST_VERSION (7, 30, 0)
 #define HAVE_CURLMOPT_MAX_TOTAL_CONNECTIONS
 #endif
+#if CURL_AT_LEAST_VERSION (7, 52, 0)
+#define HAVE_CURLOPT_PROXY_CAINFO
+#define HAVE_CURLOPT_PROXY_CAPATH
+#endif
 #if CURL_AT_LEAST_VERSION (7, 55, 0)
 #define HAVE_CURLINFO_CONTENT_LENGTH_DOWNLOAD_T
 #endif
