@@ -36,6 +36,7 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires_plugin eval
@@ -71,7 +72,7 @@ cleanup_fn rm -f $files
 # but with no sharding.
 #
 # We also need an nbdsh script that kicks off parallel writes.
-export script='
+define script <<'EOF'
 import os
 import time
 
@@ -158,12 +159,12 @@ t = end_t - start_t
 print(t, flush=True)
 assert out in [b"5"*4 + b"6"*12, b"5"*16]
 assert t >= 8.0
-'
+EOF
+export script
 
 # Now run everything
 $TRUNCATE -s 16 blocksize-sharding.img
 export witness="$PWD/blocksize-sharding.tmp"
-touch "$witness"
 nbdkit --filter=blocksize --filter=delay eval delay-write=2 \
     config='ln -sf "$(realpath "$3")" $tmpdir/$2' \
     img="$PWD/blocksize-sharding.img" tmp="$PWD/blocksize-sharding.tmp" \
