@@ -33,24 +33,27 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_filter cache
 requires_nbdsh_uri
+requires_run
 
-sock=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
-files="cache.img $sock cache.pid"
-rm -f $files
-cleanup_fn rm -f $files
+img=cache.img
+rm -f $img
+cleanup_fn rm -f $img
 
 # Create an empty base image.
 $TRUNCATE -s 128K cache.img
+export img
 
-# Run nbdkit with the caching filter.
-start_nbdkit -P cache.pid -U $sock --filter=cache file cache.img
-
-nbdsh --connect "nbd+unix://?socket=$sock" \
-      -c '
+define script <<'EOF'
 # Write some pattern data to the overlay and check it reads back OK.
+
+import os
+
+img = os.getenv("img")
+
 buf = b"abcd" * 16384
 h.pwrite(buf, 32768)
 zero = h.pread(32768, 0)
@@ -61,9 +64,14 @@ assert buf == buf2
 # Flushing should write through to the underlying file.
 h.flush()
 
-with open("cache.img", "rb") as file:
+with open(img, "rb") as file:
     zero = file.read(32768)
     assert zero == bytearray(32768)
     buf2 = file.read(65536)
     assert buf == buf2
-'
+EOF
+export script
+
+# Run nbdkit with the caching filter.
+nbdkit --filter=cache file $img \
+       --run ' nbdsh -u "$uri" -c "$script" '
