@@ -35,9 +35,23 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
-requires_tls_certificates
+
+# Does the nbdkit binary support TLS?
+if ! nbdkit --dump-config | grep -sq tls=yes; then
+    echo "$0: nbdkit built without TLS support"
+    exit 77
+fi
+
+# Did we create the PKI files?
+# Probably 'certtool' is missing.
+pkidir="pki"
+if [ ! -f "$pkidir/ca-cert.pem" ]; then
+    echo "$0: PKI files were not created by the test harness"
+    exit 77
+fi
 
 out=test-captive-tls-certificates.out
 cleanup_fn rm -f $out
