@@ -35,17 +35,30 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
-requires_tls_psk
+
+# Does the nbdkit binary support TLS?
+if ! nbdkit --dump-config | grep -sq tls=yes; then
+    echo "$0: nbdkit built without TLS support"
+    exit 77
+fi
+
+# Did we create the PSK keys file?
+# Probably 'psktool' is missing.
+if [ ! -s keys.psk ]; then
+    echo "$0: PSK keys file was not created by the test harness"
+    exit 77
+fi
 
 out=test-captive-tls-psk.out
 cleanup_fn rm -f $out
 rm -f $out
 
 LANG=C \
-nbdkit --tls=require --tls-psk="$pskfile" \
+nbdkit --tls=require --tls-psk=keys.psk \
        null \
        --run 'echo OUTPUT: "$tls_psk"' > $out
 cat $out
-grep "OUTPUT: .*/keys.psk" $out
+grep "OUTPUT: keys.psk" $out
