@@ -31,7 +31,9 @@
 # SUCH DAMAGE.
 
 source ./functions.sh
+#set -e
 set -x
+set -u
 
 # This test attempts to kill a Windows PID (it should use
 # "wine taskkill /f /pid $pid" instead).  As this requires
@@ -42,7 +44,19 @@ if is_windows; then
 fi
 
 requires nbdsh -c 'exit(not h.supports_tls())'
-requires_tls_psk
+
+# Does the nbdkit binary support TLS?
+if ! nbdkit --dump-config | grep -sq tls=yes; then
+    echo "$0: nbdkit built without TLS support"
+    exit 77
+fi
+
+# Did we create the PSK keys file?
+# Probably 'certtool' is missing.
+if [ ! -s keys.psk ]; then
+    echo "$0: PSK keys file was not created by the test harness"
+    exit 77
+fi
 
 plugin=.libs/test-disconnect-plugin.$SOEXT
 requires test -f $plugin
@@ -53,7 +67,7 @@ cleanup_fn rm -f $files
 
 # Start nbdkit with the disconnect plugin, which has delayed reads and
 # does disconnect on write based on export name.
-start_nbdkit -P disconnect-tls.pid --tls require --tls-psk="$pskfile" \
+start_nbdkit -P disconnect-tls.pid --tls require --tls-psk=keys.psk \
              -U $sock $plugin
 
 pid=`cat disconnect-tls.pid`
@@ -61,13 +75,11 @@ pid=`cat disconnect-tls.pid`
 # We can't use 'nbdsh -u "$uri" because of nbd_set_uri_allow_local_file.
 # Empty export name does soft disconnect on write; the write and the
 # pending read should still succeed, but second read attempt should fail.
-export pskfile
 nbdsh -c '
 import errno
-import os
 
 h.set_tls(nbd.TLS_REQUIRE)
-h.set_tls_psk_file(os.getenv("pskfile"))
+h.set_tls_psk_file("keys.psk")
 h.set_tls_username("qemu")
 h.connect_unix("'"$sock"'")
 
@@ -100,10 +112,9 @@ h.shutdown()
 # pending read should fail with lost connection.
 nbdsh -c '
 import errno
-import os
 
 h.set_tls(nbd.TLS_REQUIRE)
-h.set_tls_psk_file(os.getenv("pskfile"))
+h.set_tls_psk_file("keys.psk")
 h.set_tls_username("qemu")
 h.set_export_name("a")
 h.connect_unix("'"$sock"'")
