@@ -33,6 +33,7 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_plugin sh
 requires nbdsh --version
@@ -45,8 +46,7 @@ cleanup_fn rm -f $files
 
 # Create an nbdkit sh plugin which reflects the export name back to
 # the caller in the virtual device data and size.
-start_nbdkit -P export-name.pid -U $sock \
-             sh - <<'EOF'
+define plugin <<'EOF'
 case "$1" in
     open)
         # The export name is the handle.  nbdkit-sh-plugin removes
@@ -66,6 +66,8 @@ case "$1" in
 esac
 EOF
 
+start_nbdkit -P export-name.pid -U $sock sh - <<<"$plugin"
+
 # Try to read back various export names from the plugin.
 for e in "" "test" "/" "//" " " "/ " "?" "テスト" "-n" '\\' $'\n' "%%" \
          "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
