@@ -35,10 +35,16 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_nbdinfo
 requires_run
-requires_tls_certificates
+
+# Does the nbdkit binary support TLS?
+if ! nbdkit --dump-config | grep -sq tls=yes; then
+    echo "$0: nbdkit built without TLS support"
+    exit 77
+fi
 
 # RHEL 8 libnbd / nbdinfo doesn't support the tls-certificates
 # parameter in URIs, so connections always fail.  It's hard to detect
@@ -46,6 +52,14 @@ requires_tls_certificates
 # commit adding this feature was 847e0b9830, added in libnbd 1.9.5.
 requires_libnbd_version 1.10
 
+# Did we create the PKI files?
+# Probably 'certtool' is missing.
+pkidir="$PWD/pki"
+if [ ! -f "$pkidir/ca-cert.pem" ]; then
+    echo "$0: PKI files were not created by the test harness"
+    exit 77
+fi
+
 # This is expected to succeed.
 nbdkit -v --tls=require --tls-certificates="$pkidir" --tls-verify-peer \
        -D nbdkit.tls.session=1 \
