@@ -35,6 +35,8 @@
 
 source ./functions.sh
 set -e
+set -x
+set -u
 
 # Cannot use kill pidfile below to test if the process is running on
 # Windows.
@@ -44,8 +46,8 @@ if is_windows; then
 fi
 
 requires ip -V
-requires "$QEMU_IMG" --version
-requires "$QEMU_IMG" info --image-opts driver=file,filename=functions.sh
+requires qemu-img --version
+requires qemu-img info --image-opts driver=file,filename=functions.sh
 requires_ipv6_loopback
 
 rm -f ipv6lo.pid ipv6lo.out
@@ -65,7 +67,7 @@ kill -s 0 $pid
 # Check we can connect over the IPv6 loopback interface.
 ipv6_lo="$(ip -o -6 addr show scope host)"
 if test -n "$ipv6_lo"; then
-    "$QEMU_IMG" info --output=json \
+    qemu-img info --output=json \
         --image-opts "file.driver=nbd,file.host=::1,file.port=$port" > ipv6lo.out
     cat ipv6lo.out
     grep -sq '"virtual-size": *104857600\b' ipv6lo.out
