@@ -36,8 +36,9 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
-requires "$QEMU_IMG" --version
+requires qemu-img --version
 requires_nbdcopy
 requires_run
 requires $TRUNCATE --version
@@ -54,7 +55,7 @@ cleanup_fn rm -f $encrypt_disk $zero_disk
 rm -f $encrypt_disk $zero_disk
 
 # Create an empty encrypted disk container.
-"$QEMU_IMG" create -f luks \
+qemu-img create -f luks \
          --object secret,data=123456,id=sec0 \
          -o key-secret=sec0 \
          $encrypt_disk 100M
