@@ -33,10 +33,11 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_nbdinfo
 requires_run
-requires "$QEMU_IMG" --version
+requires qemu-img --version
 requires_filter luks
 
 # Test fails on macOS (darwin) because of:
@@ -52,7 +53,7 @@ info=luks-info.log
 cleanup_fn rm -f $disk $info
 rm -f $disk $info
 
-"$QEMU_IMG" create -f luks \
+qemu-img create -f luks \
          --object secret,data=123456,id=sec0 \
          -o key-secret=sec0 \
          $disk 1M
