@@ -33,11 +33,12 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires jq --version
-requires "$QEMU_IMG" --version
-requires "$QEMU_IMG" map --help
+requires qemu-img --version
+requires qemu-img map --help
 
 out="test-null-extents.out"
 expected="test-null-extents.expected"
@@ -45,14 +46,12 @@ files="$out $expected"
 rm -f $files
 cleanup_fn rm -f $files
 
-export QEMU_IMG
-
 do_test ()
 {
     # We use jq to normalize the output and convert it to plain text.
     nbdkit \
            null "$1" \
-           --run '"$QEMU_IMG" map -f raw --output=json "$uri"' |
+           --run 'qemu-img map -f raw --output=json "$uri"' |
         jq -c '.[] | {start:.start, length:.length, data:.data, zero:.zero}' \
            > $out
     if ! cmp $out $expected; then
