@@ -32,11 +32,25 @@
 
 source ./functions.sh
 set -x
+set -u
 set -e
 
 requires_plugin null
 requires_run
-requires_tls_certificates
+
+# Does the nbdkit binary support TLS?
+if ! nbdkit --dump-config | grep -sq tls=yes; then
+    echo "$0: nbdkit built without TLS support"
+    exit 77
+fi
+
+# Did we create the PKI files?
+# Probably 'certtool' is missing.
+pkidir="$PWD/pki"
+if [ ! -f "$pkidir/ca-cert.pem" ]; then
+    echo "$0: PKI files were not created by the test harness"
+    exit 77
+fi
 
 out="print-uri-tls.out"
 rm -f $out
