@@ -33,12 +33,13 @@
 source ./functions.sh
 set -e
 set -x
+set -u
 
 requires_run
 requires_plugin sh
 requires jq --version
-requires "$QEMU_IMG" --version
-requires "$QEMU_IMG" map --help
+requires qemu-img --version
+requires qemu-img map --help
 
 out="test-sh-extents.out"
 expected="test-sh-extents.expected"
@@ -46,12 +47,10 @@ files="$out $expected"
 rm -f $files
 cleanup_fn rm -f $files
 
-export QEMU_IMG
-
 do_test ()
 {
     # We use jq to normalize the output and convert it to plain text.
-    nbdkit -v sh - --run '"$QEMU_IMG" map -f raw --output=json "$uri"' |
+    nbdkit -v sh - --run 'qemu-img map -f raw --output=json "$uri"' |
         jq -c '.[] | {start:.start, length:.length, data:.data, zero:.zero}' \
            > $out
     if ! cmp $out $expected; then
