@@ -34,8 +34,10 @@
 
 source ./functions.sh
 set -e
+#set -x
+set -u
 
-# Try to get a list of tests written in shell script.  A good
+# Try to get a list of tests written in shell script.  An good
 # approximation is to see which ones include 'functions.sh'.
 tests="$( grep -l functions.sh $srcdir/test-*.sh )"
 
